﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Bulldozer.Entities;
using Bulldozer.Drawnings;
using Bulldozer.MovementStrategy;

namespace Bulldozer.DrawningObjects
{
    public class DrawningBulldozer
    {
        public EntityBulldozer? EntityTractor { get; set; }
        private int _pictureWidth;
        private int _pictureHeight;
        protected int _startPosX;
        protected int _startPosY;
        protected readonly int _tractorWidth = 160;
        protected readonly int _tractorHeight = 80;
        public DrawningBulldozer(int speed, double weight, Color mainColor, int width, int heigth)
        {
            if (width <= _tractorWidth || heigth <= _tractorHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = heigth;
            EntityTractor = new EntityBulldozer(speed, weight, mainColor);
        }
        protected DrawningBulldozer(int speed, double weight,
            Color mainColor, int width, int heigth,
            int tractorWidth, int tractorHeight)
        {
            if (width <= tractorWidth || heigth <= tractorHeight)
            {
                return;
            }
            _pictureHeight = heigth;
            _pictureWidth = width;
            _tractorHeight = tractorHeight;
            _tractorWidth = tractorWidth;
            EntityTractor = new EntityBulldozer(speed, weight, mainColor);
        }
        public void SetPosition(int x, int y)
        {
            if (x < 0 || y < 0 || x + _tractorWidth > _pictureWidth || y + _tractorHeight > _pictureHeight)
            {
                x = 10;
                y = 10;
            }
            _startPosX = x;
            _startPosY = y;
        }
        public IMoveableObject GetMoveableObject => new DrawningObjectBulldozer(this);
        public int GetPosX => _startPosX;
        public int GetPosY => _startPosY;
        public int GetWidth => _tractorWidth;
        public int GetHeight => _tractorHeight;
        public bool CanMove(DirectionTypeBulldozer direction)
        {
            if (EntityTractor == null)
            {
                return false;
            }
            return direction switch
            {
                DirectionTypeBulldozer.Left => _startPosX - EntityTractor.Step > 0,
                DirectionTypeBulldozer.Up => _startPosY - EntityTractor.Step > 0,
                DirectionTypeBulldozer.Right => _startPosX + EntityTractor.Step + _tractorWidth <= _pictureWidth,
                DirectionTypeBulldozer.Down => _startPosY + EntityTractor.Step + _tractorHeight <= _pictureHeight,
                _ => false,
            };
        }
        public void MoveTransport(DirectionTypeBulldozer direction)
        {
            if (!CanMove(direction) || EntityTractor == null)
            {
                return;
            }
            switch (direction)
            {
                case DirectionTypeBulldozer.Left:
                    _startPosX -= (int)EntityTractor.Step;
                    break;
                case DirectionTypeBulldozer.Up:
                    _startPosY -= (int)EntityTractor.Step;
                    break;
                case DirectionTypeBulldozer.Right:
                    _startPosX += (int)EntityTractor.Step;
                    break;
                case DirectionTypeBulldozer.Down:
                    _startPosY += (int)EntityTractor.Step;
                    break;
            }
        }
        public virtual void DrawTrasport(Graphics g)
        {
            if (EntityTractor == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush mainBrush = new SolidBrush(EntityTractor.MainColor);
            // Тело трактора
            Brush tractorColor = new SolidBrush(EntityTractor.MainColor);
            g.FillRectangle(tractorColor, _startPosX + 50, _startPosY + 20, 100, 30);
            g.FillRectangle(tractorColor, _startPosX + 80, _startPosY, 10, 30);
            int x = _startPosX + 50; // начальная позиция X
            int y = _startPosY; // начальная позиция Y
            int width = 110; // ширина прямоугольника
            int height = 30; // высота прямоугольника
            int radius = 20; // радиус закругления углов
            // Рисуем закругленный прямоугольник
            g.DrawArc(pen, x - 5, y + 50, radius * 2, radius * 2, 180, 90); // верхний левый угол
            g.DrawLine(pen, x + radius - 5, y + 50, x + width - radius - 5, y + 50); // верхняя горизонталь
            g.DrawArc(pen, x + width - radius * 2 - 5, y + 50, radius * 2, radius * 2, 270, 90); // верхний правый угол
            g.DrawArc(pen, x + width - radius * 2 - 5, y + height - radius * 2 + 50, radius * 2, radius * 2, 0, 90); // нижний правый угол
            g.DrawLine(pen, x + width - radius - 5, y + height + 50, x + radius - 5, y + height + 50); // нижняя горизонталь
            g.DrawArc(pen, x - 5, y + height - radius * 2 + 50, radius * 2, radius * 2, 90, 90); // нижний левый угол
            int wheelRadius = 15;
            // Рисуем колеса трактора
            g.DrawEllipse(pen, _startPosX + 50, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.FillEllipse(mainBrush, _startPosX + 50, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.DrawEllipse(pen, _startPosX + 120, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.FillEllipse(mainBrush, _startPosX + 120, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            // Кабина
            Brush cabinColor = new SolidBrush(EntityTractor.MainColor);
            g.FillRectangle(cabinColor, _startPosX + 120, _startPosY, 30, 20);
        }
    }
}
