using Bulldozer.DrawningObjects;
using Bulldozer.MovementStrategy;
using Bulldozer.Drawnings;

namespace Bulldozer
{
    public partial class FastBulldozer : Form
    {
        private DrawningBulldozer? _drawningTractor;
        private AbstractStrategy? _abstractStrategy;
        public DrawningBulldozer? SelectedTractor { get; private set; }
        public FastBulldozer()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedTractor = null;
        }
        private void Draw()
        {
            if (_drawningTractor == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxFastBulldozer.Width,
                pictureBoxFastBulldozer.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningTractor.DrawTrasport(gr);
            pictureBoxFastBulldozer.Image = bmp;
        }
        private void ButtonCreateFastBulldozer_Click(object sender, EventArgs e)
        {
            Random random = new Random();
            Color dopColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog_dop = new();
            if (dialog_dop.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialog_dop.Color;
            }
            Color dopColor2 = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog_dop2 = new();
            if (dialog_dop2.ShowDialog() == DialogResult.OK)
            {
                dopColor2 = dialog_dop2.Color;
            }
            _drawningTractor = new DrawningFastBulldozer(
                random.Next(100, 300), random.Next(1000, 3000),
                dopColor,
                dopColor2,
                Convert.ToBoolean(random.Next(0, 2)),
                Convert.ToBoolean(random.Next(0, 2)),
                pictureBoxFastBulldozer.Width,
                pictureBoxFastBulldozer.Height);
            _drawningTractor.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        private void ButtonCreateBulldozer_Click(object sender, EventArgs e)
        {
            Random random = new Random();
            Color dopColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog_dop = new();
            if (dialog_dop.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialog_dop.Color;
            }
            _drawningTractor = new DrawningBulldozer(
                random.Next(100, 300),
                random.Next(1000, 3000),
                dopColor,
                pictureBoxFastBulldozer.Width,
                pictureBoxFastBulldozer.Height);
            _drawningTractor.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningTractor == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningTractor.MoveTransport(DirectionTypeBulldozer.Up);
                    break;
                case "buttonDown":
                    _drawningTractor.MoveTransport(DirectionTypeBulldozer.Down);
                    break;
                case "buttonLeft":
                    _drawningTractor.MoveTransport(DirectionTypeBulldozer.Left);
                    break;
                case "buttonRight":
                    _drawningTractor.MoveTransport(DirectionTypeBulldozer.Right);
                    break;
            }
            Draw();
        }

        private void Buttonstep_Click(object sender, EventArgs e)
        {
            if (_drawningTractor == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex
                    switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(
                    new DrawningObjectBulldozer(_drawningTractor),
                    pictureBoxFastBulldozer.Width,
                    pictureBoxFastBulldozer.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }

        private void ButtonSelectBulldozer_Click(object sender, EventArgs e)
        {
            SelectedTractor = _drawningTractor;
            DialogResult = DialogResult.OK;
        }
    }
}