﻿
using Bulldozer.Drawnings;
using Bulldozer.Entities;
using Bulldozer;
using Bulldozer.DrawningObjects;

namespace Bulldozer
{
    /// <summary>
    /// Форма создания объекта
    /// </summary>
    public partial class FormBulldozerConfig : Form
    {
        /// <summary>
        /// Переменная-выбранная установка
        /// </summary>
        DrawningBulldozer? _bulldozer = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawningBulldozer> EventAddBulldozer;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBulldozerConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += panelColor_MouseDown;
            panelPurple.MouseDown += panelColor_MouseDown;
            panelGray.MouseDown += panelColor_MouseDown;
            panelGreen.MouseDown += panelColor_MouseDown;
            panelRed.MouseDown += panelColor_MouseDown;
            panelWhite.MouseDown += panelColor_MouseDown;
            panelYellow.MouseDown += panelColor_MouseDown;
            panelBlue.MouseDown += panelColor_MouseDown;
            // TODO buttonCancel.Click with lambda
            buttonCancel.Click += (sender, e) => Close();
        }
        /// <summary>
        /// Отрисовать установку
        /// </summary>
        private void DrawBulldozer()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _bulldozer?.SetPosition(5, 5);
            _bulldozer?.DrawTrasport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawningBulldozer> ev)
        {
            if (EventAddBulldozer == null)
            {
                EventAddBulldozer = new Action<DrawningBulldozer>(ev);
            }
            else
            {
                EventAddBulldozer += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _bulldozer = new DrawningBulldozer((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _bulldozer = new DrawningFastBulldozer((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxRearbucket.Checked,
                   checkBoxCovsh.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawBulldozer();
        }

        /// <summary>
        /// Отправляем цвет с панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void panelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        // TODO Реализовать логику смены цветов: основного и дополнительного (для продвинутого объекта)
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_bulldozer != null)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    _bulldozer.EntityTractor.MainColor = (Color)e.Data.GetData(typeof(Color));

                }
                DrawBulldozer();
            }
        }

        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (_bulldozer != null && _bulldozer.EntityTractor is EntityFastBulldozer entityfastbulldozer)
            {
                labelDopColor.AllowDrop = true;
            }
            else
                labelDopColor.AllowDrop = false;

            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_bulldozer != null && _bulldozer.EntityTractor is EntityFastBulldozer entityfastbulldozer)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    entityfastbulldozer.OptionalColor = (Color)e.Data.GetData(typeof(Color));

                }
                DrawBulldozer();
            }
        }
        /// <summary>
        /// Добавление
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddBulldozer?.Invoke(_bulldozer);
            Close();
        }
    }

}