using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Serilog;

namespace Bulldozer
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            using (ServiceProvider serviceProvider = services.BuildServiceProvider())
            {
                Application.Run(serviceProvider.GetRequiredService<FormBulldozerCollection>());
            }
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddSingleton<FormBulldozerCollection>().AddLogging(option =>
            {
                string[] path = Directory.GetCurrentDirectory().Split('\\');
                string pathNeed = "";
                for (int i = 0; i < path.Length - 3; i++)
                {
                    pathNeed += path[i] + "\\";
                }
                var configuration = new ConfigurationBuilder().SetBasePath(Directory.GetCurrentDirectory()).AddJsonFile(path: $"{pathNeed}appsettings.json", optional: false, reloadOnChange: true).Build();
                var logger = new LoggerConfiguration().ReadFrom.Configuration(configuration).CreateLogger();

                option.SetMinimumLevel(LogLevel.Information);
                option.AddSerilog(logger);
            });
        }
    }
}