﻿using Bulldozer.Exceptions;

namespace Bulldozer.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
    where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="tractor">Добавляемая установка</param>
        /// <returns></returns>
        public int Insert(T tractor)
        {
            if (_places.Count == 0)
            {
                _places.Add(tractor);
                return 0;
            }
            else
            {
                if (_places.Count < _maxCount)
                {
                    _places.Add(tractor);
                    for (int i = 0; i < _places.Count; i++)
                    {
                        T temp = _places[i];
                        _places[i] = _places[_places.Count - 1];
                        _places[_places.Count - 1] = temp;
                    }
                    return 0;
                }
                else
                {
                    throw new StorageOverflowException(_places.Count);
                }
            }
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="tractor">Добавляемая установкаь</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public bool Insert(T tractor, int position)
        {
            // TODO проверка позиции
            if (position < 0 || position >= _maxCount)
            {
                // Позиция недопустима
                throw new BulldozerNotFoundException(position);
            }
            if (Count >= _maxCount)
                throw new StorageOverflowException(position);
            _places.Insert(0, tractor);
            return true;
            
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            // TODO проверка позиции
            // Проверка позиции
            if (position < 0 || position > _maxCount || position >= Count)
                throw new BulldozerNotFoundException();
            if (_places[position] == null)
            {
                throw new BulldozerNotFoundException();
            }
            _places[position] = null;
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            // TODO проверка позиции
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                if (_places.Count <= position)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _maxCount)
                    return;
                if (_places.Count <= position)
                    return;
                _places[position] = value;
            }
        }
        /// <summary>
        /// Проход по списку
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T?> GetBulldozer(int? maxTractor = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxTractor.HasValue && i == maxTractor.Value)
                {
                    yield break;
                }
            }
        }
    }
}

